------------------------------------------------------
-- Aysel Karimova                                   --
-- SMALLSAT master thesis-PCA Jacobi                --
-- Supervisors: Kjetil Svarstad, Milica Orlandic    --
-- NTNU-EMECS-2018                                  --
-- Block RAM inference for Jacobi                   --
------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use ieee.std_logic_unsigned.all;

entity Block_RAM is
  generic(
    BRAM_size  : integer := 10000; 
    BRAM_width : integer := 32;
    ADDR_width : integer := 14
  );
  port(
    clock   : in  std_logic;
    din     : in  std_logic_vector(BRAM_width-1 downto 0);
    we      : in  std_logic; -- write enable
    address   : in  std_logic_vector(ADDR_width-1 downto 0);
    dout    : out std_logic_vector(BRAM_width-1 downto 0)
  );
end Block_RAM;

architecture behav of Block_RAM is
  --RAM declaration
  type ram_type is array (0 to BRAM_size-1) of std_logic_vector(BRAM_width-1 downto 0);
  signal RAM : ram_type;
  
  --Block RAM attribute
  attribute ram_style : string;
  attribute ram_style of RAM : signal is "block";
  
begin  
  process(clock)
  begin
    if rising_edge(clock) then
      if we = '1' then
        --write operation
        RAM(conv_integer(address)) <= din;
      else
        --read operation otherwise
        dout <= RAM(conv_integer(address));
      end if;
    end if;
  end process;
end behav;

