------------------------------------------------------
-- Aysel Karimova								                    --
-- SMALLSAT specialisation project-PCA covariance   --
-- Supervisors: Kjetil Svarstad, Milica Orlandic    --
-- NTNU-EMECS  Dec, 2017						                --
-- Updated for the master thesis, 2018              --
-- Covariance calculation - the datapath            --
------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity cov_datapath is
	generic(
		constant width_array : positive := 16;
		constant depth_array : positive := 2500;  --components, pixels
		constant planes      : positive := 100;   --equals to matrix size
		constant buffers     : positive := 4      --number of buffers
	);
	port(
		x            : in  std_logic_vector(width_array-1 downto 0);
		clock, reset : in  std_logic;
		retention    : in  std_logic;
		ctrl, enable : in  std_logic;
		y            : out signed(31 downto 0));
end cov_datapath;

architecture behav of cov_datapath is
	type buffer_array is array (depth_array - 1 downto 0) of std_logic_vector(width_array-1 downto 0);
	signal buffers_m : buffer_array;

	signal y_driver : signed(31 downto 0);
	signal mac_in   : signed(width_array-1 downto 0);
begin
	--continuous assignment
	mac_in <= signed(x) when ctrl = '1' else signed(buffers_m(depth_array - 1));
	y      <= y_driver;
	-- Clocked
	data_path : process(reset, clock)
	begin
		if reset = '1' then
			y_driver  <= (others => '0');
			buffers_m <= (others => (others => '0'));
		else
			if clock'event and clock = '1' then
				if enable = '1' then
					buffers_m <= buffers_m(depth_array-2 downto 0) & std_logic_vector(mac_in);
					if (retention = '1') then
						y_driver <= y_driver + signed(x)*mac_in;
					else
						y_driver <= signed(x)*mac_in;
					end if;
				end if;
			end if;
		end if;
	end process data_path;
end behav;
