------------------------------------------------------
-- Aysel Karimova                                   --
-- SMALLSAT master thesis-PCA Jacobi                --
-- Supervisors: Kjetil Svarstad, Milica Orlandic    --
-- NTNU-EMECS-2018                                  --
-- Error function calculation for Jacobi            --
------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.std_logic_unsigned.all;
library work;
use work.array_pkg.all;

entity off is
  generic(
    constant matrix_size  : positive := 100;
    --n is the number of computational slices
    constant n            : positive := 4 
  );
  port(
    valid_input : in std_logic;
    row_p       : in row_column_array;
    row_q       : in row_column_array;
    p           : in natural range 0 to matrix_size-1;
    q           : in natural range 0 to matrix_size-1;
    clock       : in std_logic;
    reset       : in std_logic;
    enable      : in std_logic;
    error       : out signed(71 downto 0);
    valid_output: out std_logic
  );
end entity;

architecture rtl_err of off is

  type state_type is (st_idle, st_busy);
  signal state : state_type;

  signal counter      : integer;
  signal error_signal : signed(71 downto 0);
begin
--output assignment
  error <= error_signal;

accum: process(clock, reset)
  variable error_variable   : signed(71 downto 0);
    begin
      if reset = '1' then
        error_signal <= (others => '0');
        valid_output <= '0';
        state <= st_idle;
        counter <= 0;
      else
        if clock'event and clock = '1' then
          if enable = '1' then
            
            case state is
            --pipelined calculation
               when st_busy => 
                error_variable := error_signal;
                for i in 0 to n-1 loop
                  if (n*counter+i /= p)  then
                  --multiply accumulate in Frobenius formula
                    error_variable := error_variable + row_p(n*counter + i) * row_p(n*counter + i);
                  end if;
                  if (n*counter+i /= q) then
                    error_variable := error_variable + row_q(n*counter + i) * row_q(n*counter + i);
                  end if;
                end loop;
                error_signal <= error_variable;              
                if (counter < (matrix_size/n)-1) then
                   counter <= counter + 1;
                   state <= st_busy;
                else
                   counter <= 0;
                   valid_output <= '1';
                   state <= st_idle;
                end if;
                  
               when st_idle =>
                if valid_input = '1' then
                  valid_output <= '0';
                  error_signal <= (others => '0');
                  state <= st_busy;
                else 
                  state <= st_idle;
                end if;

            end case;
            
          end if;
        end if;
      end if;
    end process;
end architecture rtl_err;