------------------------------------------------------
-- Aysel Karimova                 --
-- SMALLSAT master thesis-PCA Jacobi        --
-- Supervisors: Kjetil Svarstad, Milica Orlandic  --
-- NTNU-EMECS-2018                  --
-- Testbench for PCA algorithm            --
-- This testbench is designed for 4x4 matrix    --
------------------------------------------------------

library IEEE;
use IEEE.Std_logic_1164.all;
use IEEE.Numeric_Std.all;
use work.covariance_pkg.all;
use work.array_pkg.all;

entity toplevel_cov_bram_jac_tb is
end toplevel_cov_bram_jac_tb;

architecture sim_top of toplevel_cov_bram_jac_tb is
  constant matrix_size  : integer := 4;   --100
  constant BRAM_size    : integer := 16;  --10000
  constant BRAM_width   : integer := 32;  --32
  constant ADDR_width   : integer := 5;   --14
  constant data_width   : positive := 16; --16
  constant buffers    : positive := 2;  --4
  
  component toplevel_cov_bram_jac 
  port(
    clock     : in  std_logic;
    reset     : in  std_logic;
    enable      : in  std_logic;
    cube_data   : in  std_logic_vector(data_width - 1 downto 0);
    valid_input_cov : in  std_logic;
    output_select : in  natural range 0 to matrix_size - 1;
    select_valid  : in  std_logic;
    valid_output  : out std_logic;
    eigenvalue    : out signed(31 downto 0);
    eigenvectors  : out signed(31 downto 0)
  );
  end component;
    
  signal clock          : std_logic := '0';
  signal reset          : std_logic := '1';
  signal enable, valid_input_cov  : std_logic := '0';
  signal cube_data        : std_logic_vector(data_width-1 downto 0) := "0000000000000000";
  signal output_select      : natural range 0 to matrix_size-1;
  signal select_valid       : std_logic := '0';
  signal valid_output       : std_logic;
  signal eigenvalue       : signed(31 downto 0);
  signal eigenvectors       : signed(31 downto 0);
  
begin
  
  top : toplevel_cov_bram_jac
  port map(
    clock     => clock,
    reset     => reset,
    enable      => enable,
    valid_input_cov => valid_input_cov,
    cube_data   => cube_data,
    output_select => output_select,
    select_valid  => select_valid,
    valid_output  => valid_output,
    eigenvalue    => eigenvalue,
    eigenvectors  => eigenvectors
  );
  
  clock <= not clock after 10 ns;
    
    testing : process
      begin
      
      wait for 10 ns;
      reset <= '0';
      wait for 20 ns;
      enable <= '1';
      cube_data <= "0000000000000000";
      
      enable <= '0';
      wait until clock = '1';
      cube_data <= "0000000000000010";
      enable <= '1';
      
      valid_input_cov <= '1';
      
      for i in 0 to 64 loop
        wait until clock = '1';
        cube_data <= std_logic_vector(to_unsigned(i, 16));
      end loop;

      wait for 200 ns;
      valid_input_cov <= '0';
      
      wait for 200 ns;
      output_select <= 0;
      select_valid <= '1';
      
      if (valid_output = '1') then
        select_valid <= '0';
      end if;
      
      wait for 200 ns;
      output_select <= 1;
      select_valid <= '1';
      
      if (valid_output = '1') then
        select_valid <= '0';
      end if;       
      
      
      assert false report "test ended" severity note;
      wait; 
      end process testing;
end sim_top;